
package src;
import java.util.ArrayList;
//Abstract base class
public abstract class ActivityNode implements Runnable {
	protected String name;
	protected String id = "";
	protected Transition targetTrans;
	protected Transition sourceTrans;
	protected Semaphore sema;
	
	//Standard constructor
	public ActivityNode(){
		sema = new Semaphore(0);
		Thread t = new Thread(this);	//Initialize and start a new thread. The run-methods however
		t.start();						//are implemented in the inherited classes. Funky!
	}
		
	public String getName(){
		return name;
	}
	
	public void setName(String _n){
		name = _n;
	}
		
	public void setTargetTrans(Transition _trans){
		targetTrans = _trans;
	}
	
	public Transition getTargetTrans(){
		return targetTrans;
	}
	
	public void setSourceTrans(Transition _trans){
		sourceTrans = _trans;
	}
	
	public Transition getSourceTrans(){
		return sourceTrans;
	}
	
	public void executeAction(Transition _callerTrans){
		sema.unlock();		
	}
	
	//Required for AEA as they have no incoming transition  
	public void executeAction(){
		sema.unlock();		
	}
	
	//Required for CBAs when they have Pins. Will be overwritten in the corresponding CBA-Definition
	public ArrayList<Pin> getPinList(){
		return null;
	}
	
	//Standard run()-Method which is called when the Thread of an ActivityNode is started. 
	public void run(){
		sema.lock();		////For the first run. Semavalue of s is set to 0. Let run initially block 
		while(true){
			
			System.out.println(name + " running");
			
			//Check, if null. A node must not have a target transition
			if(targetTrans != null)
				targetTrans.fireTransition();
			sema.lock();
		}	//end of while true
	}	//end of run()
	
}	//end of class ActivityNode

//Subclasses:

class InitialNode extends ActivityNode {
	
	public InitialNode() {
		id = "InitialNode";
	}
	
	public String toString(){
		return "NAME: " + name + " ID: " + id;
	}
}


class CallBehaviorAction extends ActivityNode{
	protected ArrayList<Transition> targetList;			//A CBA can also have multiple targets
	private ArrayList<Transition> sourceList;			//And multiple source which are implicitly AND-ed together
	protected ArrayList<java.lang.Boolean> sourceFired;	//Array to check, if a source has fired
	
	public CallBehaviorAction() {
		id = "CallBehaviorAction";
		targetList = new ArrayList<Transition>();
		sourceList = new ArrayList<Transition>();
		sourceFired = new ArrayList<java.lang.Boolean>();
	}

	public String toString(){
		return "NAME: " + name + " ID: " + id;
	}
	
	/**
	 * Overrides the method of the base class. Used to add more than one target to a CBA node. 
	 */
	public void setTargetTrans(Transition _trans){
		if(!targetList.contains(_trans))
			targetList.add(_trans);
	}
	
	/**
	 * Overrides the method of the base class. Used to add more than one source to a CBA node. 
	 */
	public void setSourceTrans(Transition _trans){
		if(!sourceList.contains(_trans)) {
			sourceList.add(_trans);
			sourceFired.add(false);
		}
	}
	
	public synchronized void executeAction(Transition _callerTrans){
		
		int iIndex;
		boolean fire = true;
	
		iIndex = sourceList.indexOf(_callerTrans);	//Get the right index of the caller-Transition
		sourceFired.set(iIndex, true);				//Register the source with true
	
		//Test, if all sources have fired. If so, the variable fire is true after termination of the for-loop  
		for(int i = 0; i <= sourceList.size()-1; i++){
			fire = fire && sourceFired.get(i);
		}
		
		//If variable fire is true, let the CBA node fire
		if(fire) {
			resetRegistered();
			sema.unlock();
		}
	}
	
	//Set all values of the entries in the source to false
	public void resetRegistered(){
		//Step over all elements and reset them to false
		for(int i = 0; i<= sourceFired.size()-1; i++)
			sourceFired.set(i, false);
	}
	
	public void run(){
		sema.lock();
		while(true){
			System.out.println(name + " running");
			for(int i = 0; i<= targetList.size()-1; i++)
				targetList.get(i).fireTransition();			//Fire all target nodes
			sema.lock();
		}	//end of while(true)
	}	//end of void run()
}

class Pin {
	protected String name;
	
	public String getName(){
		return name;
	}
}

//Base classes for the Pins
class OutputPin extends Pin{
	public OutputPin(){}
	
	public void setTargetTrans(ObjectFlow of){
		System.out.println("If you can read this, an output pin method was not overwritten!");
	}
}

class InputPin extends Pin{
	public InputPin(){}
	
	public void execute(Object o){}
}


class SendSignalAction extends ActivityNode {
	protected AcSignal signal;
	
	public SendSignalAction(){
		id = "SendSignalAction";
	}
	
	public String toString(){
		return "NAME: " + name + " ID: " + id + " SIGNAL: " + signal.getName();
	}
	
	public void run(){
		sema.lock();
		while(true){
			System.out.println(name + " running");
			
			signal.send();
			//Check, if null. A node must not have a target transition
			if(targetTrans != null)
				targetTrans.fireTransition();
			sema.lock();
		}	//end of while true
	}	//end of run()
	
}


class AcceptEventAction extends ActivityNode {
	protected PollTrigger pollThread;
	protected Event ev;
	
	public AcceptEventAction(){
		id = "AcceptEventAction";
	}
		
	public String toString(){
		String tmp = "NAME: " + name + " ID: " + id + " TRIGGER: ";
		if(pollThread == null)
			tmp += ev.toString();
		else
			tmp += pollThread.toString();
		return tmp;
	}
	
	public void letRun(){
		sema.unlock();
	}
}


class DecisionNode extends ActivityNode {
	private ArrayList<Transition> targetList;
	private ArrayList<Transition> sourceList;
	
	public DecisionNode() {
		id = "DecisionNode";
		targetList = new ArrayList<Transition>();
		sourceList = new ArrayList<Transition>();
	}

	public String toString(){
		return "NAME: " + name + " ID: " + id;
	}
	
	//Overridden methods to set more than one target and source 
	public void setTargetTrans(Transition _trans){
		if(!targetList.contains(_trans))
			targetList.add(_trans);
	}
	
	public void setSourceTrans(Transition _trans){
		if(!sourceList.contains(_trans)) 
			sourceList.add(_trans);
	}
	
	//Unfortunately, we can't deal with a list here as return parameter
	public Transition getTargetTrans(){
		return null; }
	
	public Transition getSourceTrans(){
		return null; }
	
	public void run(){
		sema.lock();
		while(true){

			for(int i = 0; i<= targetList.size()-1; i++)
				targetList.get(i).fireTransition();			//Fire all target nodes
			sema.lock();
		}	//end of while(true)
	}
}


class ForkNode extends ActivityNode {
	private ArrayList<Transition> targetList;
	private ArrayList<Transition> sourceList;
	private ArrayList<Boolean> sourceFired;
	
	public ForkNode() {
		id = "ForkNode";
		targetList = new ArrayList<Transition>();
		sourceList = new ArrayList<Transition>();
		sourceFired = new ArrayList<Boolean>();
	}

	public String toString(){
		return "NAME: " + name + " ID: " + id;
	}

	/**
	 * Overrides the method of the base class. Used to add more than one target to a fork node. 
	 */
	public void setTargetTrans(Transition _trans){
		if(!targetList.contains(_trans))
			targetList.add(_trans);
	}
	
	/**
	 * Overrides the method of the base class. Used to add more than one source to a fork node. 
	 */
	public void setSourceTrans(Transition _trans){
		if(!sourceList.contains(_trans)) {
			sourceList.add(_trans);
			sourceFired.add(false);
		}
	}
	
	public Transition getTargetTrans(){
		return null; }
	
	public Transition getSourceTrans(){
		return null; }
	
	
	/**
	 * Method is called when a source transition fires.  
	 */
	public synchronized void executeAction(Transition _callerTrans){
		
		int iIndex;
		boolean fire = true;
		
		iIndex = sourceList.indexOf(_callerTrans);	//Get the right index of the caller-Transition
		sourceFired.set(iIndex, true);				//Register the source with true
	
		//Test, if all sources have fired. If so, the variable fire is true after termination of the for-loop  
		for(int i = 0; i <= sourceList.size()-1; i++){
			fire = fire && sourceFired.get(i);
		}
		
		//If variable fire is true, fire the fork node
		if(fire) {
			resetRegistered();
			sema.unlock();
		}
	}
	
	//Set all values of the entries in the source to false
	public void resetRegistered(){
		
		//Step over all elements and reset them to false
		for(int i = 0; i<= sourceFired.size()-1; i++)
			sourceFired.set(i, false);
	}
	
	public void run(){
		sema.lock();
		while(true){
			for(int i = 0; i<= targetList.size()-1; i++)
				targetList.get(i).fireTransition();
			sema.lock();
	
		}	//end of while(true)
	}	//end of run()
}




//Actual classes of the current model, generated with oAW-Xpand.

class ac0 extends InitialNode{
	public ac0(){
		name = "ac0";
	}
}

class Generate2 extends CallBehaviorAction {
	
	public ArrayList<OutputPin> outputPins;
	ArrayList<Pin> pins;

	public Generate2(){
		name = "Generate2";
		pins = new ArrayList<Pin>();
		
		outputPins = new ArrayList<OutputPin>();
		outputPins.add(new Generate2_out());
		pins.add(outputPins.get(outputPins.size()-1));
	}
	
	 
	class Generate2_out extends OutputPin {
		ac19 targetTrans;
		java.lang.Integer transportObject;
		public Generate2_out(){
			name = "Generate2_out";
		}
		
		public void execute(java.lang.Integer o){
			transportObject = o;
			System.out.println("Output Pin " + name + " with data " + o);
			targetTrans.fireTransition(transportObject);
		}
		
		public void setTargetTrans(ObjectFlow of){
			targetTrans = (ac19)of;
		}
	}
	
	public ArrayList<Pin> getPinList(){
		return pins;
	}
	
	
	
	public void run(){
		sema.lock();
		while(true){
			System.out.println(name + " running");
			
			int result = 2;
			OutputPin o = null;
			
			o = outputPins.get(0);
			((Generate2_out) o).execute(result);
			sema.lock(); 
		}	//end of while(true)
	}	//end of void run()
}	//end of class

class Mul extends CallBehaviorAction {
	public ArrayList<InputPin> inputPins;
	public ArrayList<OutputPin> outputPins;
	ArrayList<Pin> pins;

	public Mul(){
		name = "Mul";
		pins = new ArrayList<Pin>();
		inputPins = new ArrayList<InputPin>();
		inputPins.add(new Mul_in1());
		pins.add(inputPins.get(inputPins.size()-1));
		sourceFired.add(false);
		inputPins.add(new Mul_in2());
		pins.add(inputPins.get(inputPins.size()-1));
		sourceFired.add(false);
		outputPins = new ArrayList<OutputPin>();
		outputPins.add(new Mul_out());
		pins.add(outputPins.get(outputPins.size()-1));
	}
	
	class Mul_in1 extends InputPin {
		java.lang.Integer transportObject;
		public Mul_in1(){
			name = "Mul_in1";
		}
		
		public void execute(java.lang.Integer o){
			transportObject = o;
			executeAction(this);
		}
		
		public java.lang.Integer getTransportObject(){
			return transportObject;
		}
	}
	class Mul_in2 extends InputPin {
		java.lang.Integer transportObject;
		public Mul_in2(){
			name = "Mul_in2";
		}
		
		public void execute(java.lang.Integer o){
			transportObject = o;
			executeAction(this);
		}
		
		public java.lang.Integer getTransportObject(){
			return transportObject;
		}
	}
	 
	class Mul_out extends OutputPin {
		ac18 targetTrans;
		java.lang.Integer transportObject;
		public Mul_out(){
			name = "Mul_out";
		}
		
		public void execute(java.lang.Integer o){
			transportObject = o;
			System.out.println("Output Pin " + name + " with data " + o);
			targetTrans.fireTransition(transportObject);
		}
		
		public void setTargetTrans(ObjectFlow of){
			targetTrans = (ac18)of;
		}
	}
	
	public ArrayList<Pin> getPinList(){
		return pins;
	}
	
	public synchronized void executeAction(InputPin in){
		
		boolean fired = true;
		int index = inputPins.indexOf(in);
		sourceFired.set(index, true);
		
		for(int i = 0; i<= inputPins.size()-1; i++)
			fired = fired && sourceFired.get(i);
			
		if(fired){
			resetRegistered();
			sema.unlock();
		}
	}
	
	//Set all values of the entries in the source to false
	public void resetRegistered(){
		
		//Step over all elements and reset them to false
		for(int i = 0; i<= sourceFired.size()-1; i++)
			sourceFired.set(i, false);
	}
	
	public void run(){
		sema.lock();
		while(true){
			System.out.println(name + " running");
			
			int result = 1;					
			result *= ((Mul_in1)inputPins.get(0)).getTransportObject();
								
			result *= ((Mul_in2)inputPins.get(1)).getTransportObject();
			
			OutputPin o = null;
			
			o = outputPins.get(0);
			((Mul_out) o).execute(result);
			sema.lock(); 
		}	//end of while(true)
	}	//end of void run()
}	//end of class

class Generate5 extends CallBehaviorAction {
	
	public ArrayList<OutputPin> outputPins;
	ArrayList<Pin> pins;

	public Generate5(){
		name = "Generate5";
		pins = new ArrayList<Pin>();
		
		outputPins = new ArrayList<OutputPin>();
		outputPins.add(new Generate5_out());
		pins.add(outputPins.get(outputPins.size()-1));
	}
	
	 
	class Generate5_out extends OutputPin {
		ac20 targetTrans;
		java.lang.Integer transportObject;
		public Generate5_out(){
			name = "Generate5_out";
		}
		
		public void execute(java.lang.Integer o){
			transportObject = o;
			System.out.println("Output Pin " + name + " with data " + o);
			targetTrans.fireTransition(transportObject);
		}
		
		public void setTargetTrans(ObjectFlow of){
			targetTrans = (ac20)of;
		}
	}
	
	public ArrayList<Pin> getPinList(){
		return pins;
	}
	
	
	
	public void run(){
		sema.lock();
		while(true){
			System.out.println(name + " running");
			
			int result = 5;
			OutputPin o = null;
			
			o = outputPins.get(0);
			((Generate5_out) o).execute(result);
			sema.lock(); 
		}	//end of while(true)
	}	//end of void run()
}	//end of class

class Display extends CallBehaviorAction {
	public ArrayList<InputPin> inputPins;
	
	ArrayList<Pin> pins;

	public Display(){
		name = "Display";
		pins = new ArrayList<Pin>();
		inputPins = new ArrayList<InputPin>();
		inputPins.add(new Display_in());
		pins.add(inputPins.get(inputPins.size()-1));
		sourceFired.add(false);
		
	}
	
	class Display_in extends InputPin {
		java.lang.Integer transportObject;
		public Display_in(){
			name = "Display_in";
		}
		
		public void execute(java.lang.Integer o){
			transportObject = o;
			executeAction(this);
		}
		
		public java.lang.Integer getTransportObject(){
			return transportObject;
		}
	}
	
	
	public ArrayList<Pin> getPinList(){
		return pins;
	}
	
	public synchronized void executeAction(InputPin in){
		
		boolean fired = true;
		int index = inputPins.indexOf(in);
		sourceFired.set(index, true);
		
		for(int i = 0; i<= inputPins.size()-1; i++)
			fired = fired && sourceFired.get(i);
			
		if(fired){
			resetRegistered();
			sema.unlock();
		}
	}
	
	//Set all values of the entries in the source to false
	public void resetRegistered(){
		
		//Step over all elements and reset them to false
		for(int i = 0; i<= sourceFired.size()-1; i++)
			sourceFired.set(i, false);
	}
	
	public void run(){
		sema.lock();
		while(true){
			System.out.println(name + " running");
			
					
			System.out.println("Value of Pin Display_in is " + ((Display_in)inputPins.get(0)).getTransportObject());
			
			for(int i = 0; i<= targetList.size()-1; i++)
				targetList.get(i).fireTransition();			//Fire all target nodes
			
			sema.lock(); 
		}	//end of while(true)
	}	//end of void run()
}	//end of class

class Generate7 extends CallBehaviorAction {
	
	public ArrayList<OutputPin> outputPins;
	ArrayList<Pin> pins;

	public Generate7(){
		name = "Generate7";
		pins = new ArrayList<Pin>();
		
		outputPins = new ArrayList<OutputPin>();
		outputPins.add(new Generate7_out());
		pins.add(outputPins.get(outputPins.size()-1));
	}
	
	 
	class Generate7_out extends OutputPin {
		ac21 targetTrans;
		java.lang.Integer transportObject;
		public Generate7_out(){
			name = "Generate7_out";
		}
		
		public void execute(java.lang.Integer o){
			transportObject = o;
			System.out.println("Output Pin " + name + " with data " + o);
			targetTrans.fireTransition(transportObject);
		}
		
		public void setTargetTrans(ObjectFlow of){
			targetTrans = (ac21)of;
		}
	}
	
	public ArrayList<Pin> getPinList(){
		return pins;
	}
	
	
	
	public void run(){
		sema.lock();
		while(true){
			System.out.println(name + " running");
			
			int result = 7;
			OutputPin o = null;
			
			o = outputPins.get(0);
			((Generate7_out) o).execute(result);
			sema.lock(); 
		}	//end of while(true)
	}	//end of void run()
}	//end of class

class Add extends CallBehaviorAction {
	public ArrayList<InputPin> inputPins;
	public ArrayList<OutputPin> outputPins;
	ArrayList<Pin> pins;

	public Add(){
		name = "Add";
		pins = new ArrayList<Pin>();
		inputPins = new ArrayList<InputPin>();
		inputPins.add(new Add_in1());
		pins.add(inputPins.get(inputPins.size()-1));
		sourceFired.add(false);
		inputPins.add(new Add_in2());
		pins.add(inputPins.get(inputPins.size()-1));
		sourceFired.add(false);
		outputPins = new ArrayList<OutputPin>();
		outputPins.add(new Add_out());
		pins.add(outputPins.get(outputPins.size()-1));
	}
	
	class Add_in1 extends InputPin {
		java.lang.Integer transportObject;
		public Add_in1(){
			name = "Add_in1";
		}
		
		public void execute(java.lang.Integer o){
			transportObject = o;
			executeAction(this);
		}
		
		public java.lang.Integer getTransportObject(){
			return transportObject;
		}
	}
	class Add_in2 extends InputPin {
		java.lang.Integer transportObject;
		public Add_in2(){
			name = "Add_in2";
		}
		
		public void execute(java.lang.Integer o){
			transportObject = o;
			executeAction(this);
		}
		
		public java.lang.Integer getTransportObject(){
			return transportObject;
		}
	}
	 
	class Add_out extends OutputPin {
		ac22 targetTrans;
		java.lang.Integer transportObject;
		public Add_out(){
			name = "Add_out";
		}
		
		public void execute(java.lang.Integer o){
			transportObject = o;
			System.out.println("Output Pin " + name + " with data " + o);
			targetTrans.fireTransition(transportObject);
		}
		
		public void setTargetTrans(ObjectFlow of){
			targetTrans = (ac22)of;
		}
	}
	
	public ArrayList<Pin> getPinList(){
		return pins;
	}
	
	public synchronized void executeAction(InputPin in){
		
		boolean fired = true;
		int index = inputPins.indexOf(in);
		sourceFired.set(index, true);
		
		for(int i = 0; i<= inputPins.size()-1; i++)
			fired = fired && sourceFired.get(i);
			
		if(fired){
			resetRegistered();
			sema.unlock();
		}
	}
	
	//Set all values of the entries in the source to false
	public void resetRegistered(){
		
		//Step over all elements and reset them to false
		for(int i = 0; i<= sourceFired.size()-1; i++)
			sourceFired.set(i, false);
	}
	
	public void run(){
		sema.lock();
		while(true){
			System.out.println(name + " running");
			
			int result = 0;					
			result += ((Add_in1)inputPins.get(0)).getTransportObject();
								
			result += ((Add_in2)inputPins.get(1)).getTransportObject();
			
			OutputPin o = null;
			
			o = outputPins.get(0);
			((Add_out) o).execute(result);
			sema.lock(); 
		}	//end of while(true)
	}	//end of void run()
}	//end of class


class Send_Enable extends SendSignalAction{
	public Send_Enable(AcSignal _sig){
		name = "Send_Enable";
		signal = _sig;
	}
}

class ac1 extends AcceptEventAction{
	
	public ac1(){
		name = "ac1";
		ev = new TimeEvent(1);
	}
	
	//Is called when the source Transition fires. This is only the case for time events
	public synchronized void executeAction(){
		pollThread = new PollTrigger((TimeEvent)ev, this);
		Thread t1 = new Thread(pollThread);
		t1.start();
	}
	
	//As we don't know, if the Transition fires with parameter, we implement BOTH methods!!!
	public synchronized void executeAction(Transition trans){
		pollThread = new PollTrigger((TimeEvent)ev, this);
		Thread t1 = new Thread(pollThread);
		t1.start();
	}
	
}
class Receive_Enable extends AcceptEventAction{
	
	public Receive_Enable(ArrayList<AcSignal> signals){
		name = "Receive_Enable";
		ev = new ReceiveSignalEvent(getSignalWithName(signals, "enable"));
		pollThread = new PollTrigger((ReceiveSignalEvent)ev, this);
		Thread t1 = new Thread(pollThread);
		t1.start();
	}
	
	//Method to get the right signal (identified by name) from the signal list
	public AcSignal getSignalWithName(ArrayList<AcSignal> signals, String signalName){
		AcSignal tmp = null;
		for(int i = 0; i<= signals.size()-1; i++){
			if (signals.get(i).getName().compareTo(signalName) == 0)
				tmp = signals.get(i);
		}
		return tmp;
	}
	
}

class fork_ac6 extends ForkNode{
	public fork_ac6(){
		name = "fork_ac6";
	}
}


class dec_ac4 extends DecisionNode{
	public dec_ac4(){
		name = "dec_ac4";
	}
}
class dec_ac5 extends DecisionNode{
	public dec_ac5(){
		name = "dec_ac5";
	}
}



class Semaphore{
//Simple semaphore mechanism to prevent multiple sources to access the execute-method simultaneously 
	int semaValue = 1;	//Indicates, how many threads are allowed to access the executeAction-method at the same time. Default is 1.
	
	public Semaphore(int i){
		semaValue = i;
	}
	
	public synchronized void lock(){
		while(semaValue == 0){
			try {
				wait();	//A thread stops here, if the semaphore is locked
			} catch (InterruptedException e) {}
		}
		semaValue--;
	}
	
	public synchronized void unlock(){
		semaValue++;
		notify();	//Release the waiting threads
	}
}

